package com.mgh.pmdm.parques.view.ui

import android.app.Activity
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.provider.MediaStore
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Toast
import androidx.activity.result.ActivityResult
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.content.FileProvider
import androidx.core.widget.doAfterTextChanged
import androidx.fragment.app.activityViewModels
import androidx.lifecycle.ViewModelProvider
import com.google.android.material.snackbar.Snackbar
import com.mgh.pmdm.parques.R
import com.mgh.pmdm.parques.databinding.FragmentSecondBinding
import com.mgh.pmdm.parques.db.Park
import com.mgh.pmdm.parques.view.dialogs.MiDialogFragment
import com.mgh.pmdm.parques.viewmodel.MainViewModel
import java.io.File
import java.io.IOException


class SecondFragment : Fragment(), MiDialogFragment.OnOKOrCancelListener {

    /*********************
     * Cargadores        *
     ********************/

    private val cargadorImagen = registerForActivityResult(
        ActivityResultContracts.StartActivityForResult()
    ) { result: ActivityResult ->
        val resultCode = result.resultCode
        val resultData: Intent? = result.data

        if (resultCode== Activity.RESULT_OK){
            if (resultData != null) {
                val uri: Uri? = resultData.data

                try {
                    // Establecemos la URI obtenida como en la imagen
                    binding.Image.setImageURI(uri)
                    // Y actualizamos la URI de la imagen actual
                    viewModel.currentPark.value?.img=uri.toString()?:""

                } catch (e: IOException) {
                    e.printStackTrace()
                }

            }

        }
    }

    private val cargadorCamara = registerForActivityResult(
        ActivityResultContracts.TakePicture()
    ) { result: Boolean ->
        if (result) {
            binding.Image.setImageURI(latestTmpUri)
            tmpUri = latestTmpUri.toString()
            viewModel.currentPark.value?.img=tmpUri;
        }
    }

    // Cargador para comprobar los permisos para acceder a la cámara
    // y utilizar el almacenamiento del dispositivo.
    private val cameraPermissionRequest = registerForActivityResult(
        ActivityResultContracts.RequestMultiplePermissions()
    ) { permissions ->
        if (permissions.getOrDefault(android.Manifest.permission.CAMERA, false) &&
            permissions.getOrDefault(android.Manifest.permission.WRITE_EXTERNAL_STORAGE, false)
        )
        // Si tenemos permisos invocamos al método takePhoto()
            takePhoto()
        else {
            // Si no, mostramos una advertencia
            Log.w("[ Permisos de cámera ]", "Sin permisos")
        }
    }


    /********************+
     *  Fin Cargadores   *
     ********************/


    // Definimos el binging
    private var _binding: FragmentSecondBinding? = null
    private val binding get() = _binding!!

    // Referencia al ViewModel
    private lateinit var viewModel: MainViewModel


    // Path donde almacenar la última imagen capturada, en formato
    // URI y Texto para guardarla
    private var latestTmpUri: Uri? = null
    private var tmpUri = ""


    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Preparamos únicamente la vista
        _binding = FragmentSecondBinding.inflate(inflater, container, false)
        populateSpinner()
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Instanciamos el ViewModel, utilizando la clase ViewModelProvider
        // En lugar de this, vamos a utilizar requireActivity(), para trabajar en el ámbito
        // de la actividad y poder compartir el ViewModel
        viewModel = ViewModelProvider(requireActivity()).get(MainViewModel::class.java)

        // Creamos un nuevo método para preparar todos los
        // observers y cómo se actualizan.
        prepareObservers()

        binding.Image.setOnClickListener {
            // Cuando se hace click sobre la imagen, cargamos el
            // cargador para comprobar los permisos de la cámara
            cameraPermissionRequest.launch(
                arrayOf(
                    android.Manifest.permission.CAMERA,
                    android.Manifest.permission.WRITE_EXTERNAL_STORAGE
                )
            )
        }

        binding.Image.setOnLongClickListener {
            // Cuando se hace click sobre la imagen, cargamos el
            // cargador para comprobar los permisos de la cámara
            val openDocumentIntent = Intent(Intent.ACTION_OPEN_DOCUMENT)
            openDocumentIntent.type = "image/*"
            cargadorImagen.launch(openDocumentIntent)
            false

        }

        // Gestor del evento onClick sobre el botón de guardar
        binding.btSave.setOnClickListener {
            // Preparamos el diálogo de confirmación
            val miDialogo = MiDialogFragment(
                resources.getString(R.string.SaveData),
                resources.getString(R.string.AskSaveData),
                this
            )
            miDialogo.show(requireActivity().supportFragmentManager, "miDialogo")
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    /*********************************
     * Métodos específicos de la clase
     **********************************/

    fun prepareObservers() {
        // Definición de observadores a los LiveData para ser notificados de los cambios

        // Observer para el parque en edición:
        // Actualizamos las vistas, y al mismo tiempo, cuando se modifican
        // actualizamos el current_park
        viewModel.currentPark.observe(viewLifecycleOwner) {
            it?.let { mypark ->

                // ImageView
                if (mypark.img!="") binding.Image.setImageURI(Uri.parse(mypark.img))
                else binding.Image.setImageResource(R.drawable.appimg)

                //  TextViews
                binding.Name.setText(mypark.name)
                binding.Name.doAfterTextChanged { mypark.name = it.toString() }
                binding.Description.setText(mypark.desc)
                binding.Description.doAfterTextChanged { mypark.desc = it.toString() }
                binding.Phone.setText(mypark.phone)
                binding.Phone.doAfterTextChanged { mypark.phone = it.toString() }
                binding.webSite.setText(mypark.webiste)
                binding.webSite.doAfterTextChanged { mypark.webiste = it.toString() }

                // Actualización del valor de los spinners
                for (i in 0..binding.SpinnerOpeningTime.adapter.count)
                    if (binding.SpinnerOpeningTime.adapter.getItem(i).equals(mypark.openingTime)) {
                        binding.SpinnerOpeningTime.setSelection(i)
                        break
                    }

                // Listeners para los cambios en los spinners
                binding.SpinnerOpeningTime.onItemSelectedListener = object :
                    AdapterView.OnItemSelectedListener {
                    override fun onItemSelected(
                        parent: AdapterView<*>?,
                        view: View?,
                        position: Int,
                        id: Long
                    ) {
                        it.openingTime = parent?.getItemAtPosition(position).toString()
                    }

                    override fun onNothingSelected(parent: AdapterView<*>?) {
                        // Nada por hacer
                    }
                }

                for (i in 0..binding.SpinnerClosingTime.adapter.count)
                    if (binding.SpinnerClosingTime.adapter.getItem(i).equals(it.closingTime)) {
                        binding.SpinnerClosingTime.setSelection(i)
                        break
                    }

                binding.SpinnerClosingTime.onItemSelectedListener = object :
                    AdapterView.OnItemSelectedListener {
                    override fun onItemSelected(
                        parent: AdapterView<*>?,
                        view: View?,
                        position: Int,
                        id: Long
                    ) {
                        it.closingTime = parent?.getItemAtPosition(position).toString()
                    }

                    override fun onNothingSelected(parent: AdapterView<*>?) {
                        // Nada por hacer
                    }
                }

                // Actualización de los Checkboxes
                if (mypark.sports == true) binding.cbSport.isChecked = true

                binding.cbSport.setOnCheckedChangeListener { _, isChecked ->
                    mypark.sports = isChecked
                }

                if (mypark.children == true) binding.cbChildren.isChecked = true

                binding.cbChildren.setOnCheckedChangeListener { _, isChecked ->
                    mypark.children = isChecked
                }

                if (mypark.Bar == true) binding.cbBar.isChecked = true
                binding.cbBar.setOnCheckedChangeListener { _, isChecked ->
                    mypark.Bar = isChecked
                }

                if (mypark.Pets == true) binding.cbMascotas.isChecked = true
                binding.cbMascotas.setOnCheckedChangeListener { _, isChecked ->
                    mypark.Pets = isChecked
                }

            }

        }


        // Observa cuando se ha añadido un nuevo parque para mostrar el Snackbar
        viewModel.parkSaved.observe(viewLifecycleOwner) { saved ->
            saved?.let {
                // Limpiamos el valor para no entrar en un bucle infinito
                viewModel.parkSaved.value = null
                Snackbar.make(
                    binding.root,
                    resources.getString(R.string.dataSaved),
                    Snackbar.LENGTH_LONG
                ).show()
            }
        }

        // Observa cuando se ha modificado un parque para mostrar el Snackbar
        viewModel.parkUpdated.observe(viewLifecycleOwner) { saved ->
            saved?.let {
                // Limpiamos el valor para no entrar en un bucle infinito
                viewModel.parkUpdated.value = null
                Snackbar.make(
                    binding.root,
                    resources.getString(R.string.updatedSaved),
                    Snackbar.LENGTH_LONG
                ).show()
            }
        }
    }

    // Método específico para poblar el spinner para la hora de cierre
    private fun populateSpinner() {
        ArrayAdapter.createFromResource(
            requireActivity(),
            R.array.horas,
            android.R.layout.simple_spinner_dropdown_item
        ).also { adapter ->
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_item)
            binding.SpinnerClosingTime.adapter = adapter
        }
    }

    override fun onPositiveClick() {
        // Creamos un nuevo parque con la información del formulario
        val newPark = Park(
            name = binding.Name.text.toString(),
            desc = binding.Description.text.toString(),
            phone = binding.Phone.text.toString(),
            webiste = binding.webSite.text.toString(),
            openingTime = binding.SpinnerOpeningTime.selectedItem.toString(),
            closingTime = binding.SpinnerClosingTime.selectedItem.toString(),
            sports = binding.cbSport.isChecked,
            children = binding.cbChildren.isChecked,
            Bar = binding.cbBar.isChecked,
            Pets = binding.cbMascotas.isChecked,
            img = tmpUri?:""
        )

        viewModel.savePark(newPark)

    }


    override fun onCancelClick() {
        // Si pulsa cancelar mostramos un toast
        Toast.makeText(
            requireActivity().applicationContext,
            resources.getString(R.string.actionCancelled), Toast.LENGTH_SHORT
        ).show()
    }


    // Método para tomar una fotografía
    private fun takePhoto() {
        // Obtenemos la carpeta de almacenamiento externo de la aplicación, mediante
        // el método getExternalFilesDir de la actividad
        val dir = requireActivity().getExternalFilesDir(Environment.DIRECTORY_PICTURES)

        // Generamos el fichero temporal
        val file = File.createTempFile("parques", ".jpg", dir)

        // Obtenemos su URI, almacenándola en la variable latestTmpUri, definida
        // como variable miembro de la clase
        latestTmpUri =
            FileProvider.getUriForFile(
                requireActivity(),
                requireActivity().applicationContext.packageName + ".provider",
                file
            )

        // Y proporcionamos esta ruta al cargador de la cámara, para
        // que guarde la imagen capturada en la URI especificada.

        cargadorCamara.launch(latestTmpUri)
    }


}